using System;
using System.Collections.Generic;
using System.Text;

using System.Data;
using System.IO;
using System.Xml;

using TaskPluginInterface;
using Schedule;

namespace ApplicationPlugins
{
    [TaskPluginInterface.Plugin(PluginType.Executable)]
    public class FileCopy : ITaskPlugin
    {

        #region Private Variables

        // For Properties
        private List<IScheduledItem> m_Schedule;

        // For configurtion
        private FileCopyConfig config;
        private FileCopyConfig.ConfigDirectory[] m_Dirs;

        // For Progress
        private Utilities.Progress m_Progress;

        // Other Variables
        private log4net.ILog m_Log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        // Cancel button was clicked
        private bool m_Cancel = false;

        #region Private Constants


        #endregion Private Constants

        #region Private Enums

        #endregion Private Enums

        #endregion Private Variables

        #region Constructors

        public FileCopy()
        {

            LoadConfiguration();

        }

        #endregion Constructors

        #region ITaskPlugin Members

        #region ITaskPlugin Properties

        public string Name
        {
            get { return Properties.FileCopyPlugin.PluginName; }
        }

        public string Description
        {
            get { return Properties.FileCopyPlugin.PluginDescription; }
        }

        public string Extension
        {
            get { return Properties.FileCopyPlugin.PluginExtension; }
        }

        public string ExtensionDescription
        {
            get { return Properties.FileCopyPlugin.PluginExtensionDescription; }
        }

        public string Version
        {
            get { return Properties.FileCopyPlugin.PluginVersion; }
        }

        public string Author
        {
            get { return Properties.FileCopyPlugin.PluginAuthor; }
        }

        public List<IScheduledItem> Schedule
        {
            get { return m_Schedule; }
        }

        #endregion ITaskPlugin Properties

        #region ITaskPlugin Methods

        public ExecuteResult Execute()
        {

            #region Variables

            #endregion Variables

            #region Data Validation

            #endregion Data Validation

            try
            {
                // Setup the Progress Information
                m_Progress.TotalTasks = 1;
                m_Progress.TotalItems = 1;
                m_Progress.CurrentTask = 0;
                m_Progress.CurrentItem = 0;
                m_Cancel = false;

                m_Log.Debug(Properties.Resources.Execute_Started);

                // If the EventStart event has been subscribed to let's fire it.
                if (EventStart != null)
                    EventStart(this, new PluginEventArgs(m_Progress, "Starting the task.", null));

                DoWork();

                // If the EventStart event has been subscribed to let's fire it.
                if (EventEnd != null)
                {
                    if (m_Cancel)
                    {
                        EventEnd(this, new PluginEventArgs(m_Progress, "Task execution was cancelled.", null));
                    }
                    else
                    {
                        EventEnd(this, new PluginEventArgs(m_Progress, "Task execution is complete.", null));
                    }
                }

                // Exit 
                if (m_Cancel)
                {
                    m_Log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Cancelled);
                    return ExecuteResult.Cancelled;
                }
                else
                {
                    m_Log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Ok);
                    return ExecuteResult.Ok;
                }
            }
            catch (Exception ex)
            {
                if (EventExceptionOccurred != null)
                    EventExceptionOccurred(this, new PluginEventArgs(m_Progress, "Exception Raised", ex));

                // Exit with an Exception
                return ExecuteResult.Exception;
            }

        }

        public bool Cancel()
        {
            m_Log.Debug(Properties.Resources.CancelMethodExecuted);
            m_Cancel = true;
            return true;
        }
        #endregion ITaskPlugin Methods

        #region ITaskPlugin Events
        public event OnStart EventStart;
        public event OnEnd EventEnd;
        public event OnProcessing EventProcessing;
        public event OnExceptionOccurred EventExceptionOccurred;
        #endregion ITaskPlugin Events

        #endregion

        #region Private Methods

        /// <summary>
        /// This method reads the "Directories" from the configuration file and copies
        /// the files.
        /// </summary>
        /// <returns>An <see cref="TaskPluginInterface.ExecuteResult"/> value, with the results of
        /// the method.</returns>
        private ExecuteResult DoWork()
        {

            // Reset the currentItem counter
            m_Progress.CurrentItem = 0;
            m_Progress.CurrentTask = 0;

            // Set the numbers of items to process
            m_Progress.TotalTasks = m_Dirs.Length;

            m_Log.DebugFormat(Properties.Resources.FoundDirectories, m_Dirs.Length);

            foreach (FileCopyConfig.ConfigDirectory configDir in m_Dirs)
            {
                m_Progress.CurrentItem = 0;

                // Get the file list that matches the FileMask
                string[] files = Directory.GetFiles(configDir.FromDir, configDir.FileMask);
                // Update the Total Items count
                m_Progress.TotalItems = files.Length;

                m_Log.DebugFormat(Properties.Resources.FoundFiles, files.Length, configDir.FromDir, configDir.FileMask);

                // See if we need to create the new directory
                if (Directory.Exists(configDir.ToDir) == false)
                {
                    m_Log.DebugFormat(Properties.Resources.CreatingDirectory, configDir.ToDir);
                    Directory.CreateDirectory(configDir.ToDir);
                }

                // Loop through each file.
                foreach (string file in files)
                {
                    string msg = string.Format(Properties.Resources.CopyingFile, file, configDir.FromDir, configDir.ToDir);

                    // Create the new filename
                    FileInfo copyFile = new FileInfo(file);

                    // get the name of the new file.
                    string newFile = Path.Combine(configDir.ToDir, copyFile.Name);
                    copyFile.CopyTo(newFile, true);

                    m_Progress.CurrentItem++;
                    if (EventProcessing != null)
                    {
                        PluginEventArgs evt = new PluginEventArgs(m_Progress, msg, null);
                        EventProcessing(this, evt);
                        if (evt.Cancel == true)
                        {
                            m_Log.Debug(Properties.Resources.CancelRequestRecieved);
                            m_Cancel = true;
                            break;
                        }
                    }
                }

                if (m_Cancel)
                    break;

                if (EventProcessing != null)
                {
                    PluginEventArgs evt = new PluginEventArgs(m_Progress, "Working", null);
                    EventProcessing(this, evt);
                    if (evt.Cancel == true)
                    {
                        m_Log.Debug(Properties.Resources.CancelRequestRecieved);
                        m_Cancel = true;
                        break;
                    }
                }

                m_Progress.CurrentTask++;
            }

            if (m_Cancel)
                return ExecuteResult.Cancelled;
            else
                return ExecuteResult.Ok;

        }

        #endregion Private Methods

        #region Helper Functions

        #region LoadConfiguration
        private void LoadConfiguration()
        {

            string configFile = System.Reflection.Assembly.GetExecutingAssembly().Location + ".Settings.xml";
            config = new FileCopyConfig(null, configFile);

            // TODO: Figure out how to have different log4net instances for each
            //      plug-in.
            // Commented out until a fix can be determined to seperate the log files.
            // for each plug-in
            //if (!string.IsNullOrEmpty(config.Log4NetConfig))
            //{
            //    XmlDocument xml = new XmlDocument();
            //    xml.LoadXml(config.Log4NetConfig);
            //    log4net.Config.XmlConfigurator.Configure(xml.DocumentElement);
            //    xml = null;
            //}

            m_Schedule = TaskPluginInterface.ScheduleConfig.Load(config.Schedule);

            m_Dirs = config.Directories;

            m_Log.Debug(Properties.Resources.ConfigurationLoaded);
        }
        #endregion LoadConfiguration

        #endregion Helper Functions
    }
}
