using System;
using System.Xml;
using System.Collections.Generic;
using Guadagno.Utilities.Configuration;

namespace ApplicationPlugins
{
    public class FileCopyConfig : ConfigurationBase
    {
        public FileCopyConfig() : base() { }
        public FileCopyConfig(string stage) : base(stage) { }
        public FileCopyConfig(string stage, string filename) : base(stage, filename) { }

        #region Public Properties From XML

        public string Schedule { get { return this["Schedule"]; } }
        public string Log4NetConfig { get { return this["log4net"]; } }

        public ConfigDirectory[] Directories
        {
            get
            {
                List<ConfigDirectory> dirs = new List<ConfigDirectory>();

                if (!string.IsNullOrEmpty(this["Directories"]))
                {
                    XmlDocument xml = new XmlDocument();
                    xml.LoadXml(this["Directories"]);
                    XmlNode directories = xml.SelectSingleNode("Directories");

                    foreach (XmlNode dir in directories.ChildNodes)
                    {
                        if (dir.NodeType != XmlNodeType.Element) continue;

                        ConfigDirectory cDir = new ConfigDirectory();
                        cDir.FromDir = getAttribValue(dir,"FromDir", "");
                        cDir.ToDir = getAttribValue(dir,"ToDir", "");
                        cDir.FileMask = getAttribValue(dir, "FileMask", "");
                        dirs.Add(cDir);
                    }

                    xml = null;
                }
                return dirs.ToArray();
            }
        }

        #endregion Public Properties From XML

        #region ConfigDirectory Struct
        public struct ConfigDirectory
        {
            public string FromDir;
            public string ToDir;
            public string FileMask;
        }
        #endregion ConfigDirectory Struct
    }
}
