using System;
using System.Collections.Generic;
using System.Text;

using System.Diagnostics;
using System.Xml;

using TaskPluginInterface;
using Schedule;

namespace ApplicationPlugins
{
    [TaskPluginInterface.Plugin(PluginType.Executable)]
    public class RunCommand : ITaskPlugin
    {

        #region Private Variables

        // For Properties
        private List<IScheduledItem> m_Schedule;

        // For configuration
        private RunCommandConfig config;

        // For Progress
        private Utilities.Progress m_Progress;

        // Other Variables
        private log4net.ILog m_Log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        // Cancel button was clicked
        private bool m_Cancel = false;

        #region Private Enums

        #endregion Private Enums

        #region Constants

        #endregion Constants

        #endregion Private Variables

        #region Constructors

        public RunCommand()
        {

            LoadConfiguration();

        }

        #endregion Constructors

        #region ITaskPlugin Members

        #region ITaskPlugin Properties

        public string Name
        {
            get { return Properties.RunCommandPlugin.PluginName; }
        }

        public string Description
        {
            get { return Properties.RunCommandPlugin.PluginDescription; }
        }

        public string Extension
        {
            get { return Properties.RunCommandPlugin.PluginExtension; }
        }

        public string ExtensionDescription
        {
            get { return Properties.RunCommandPlugin.PluginExtensionDescription; }
        }

        public string Version
        {
            get { return Properties.RunCommandPlugin.PluginVersion; }
        }

        public string Author
        {
            get { return Properties.RunCommandPlugin.PluginAuthor; }
        }

        public List<IScheduledItem> Schedule
        {
            get { return m_Schedule; }
        }

        #endregion ITaskPlugin Properties

        #region ITaskPlugin Methods

        public ExecuteResult Execute()
        {

            #region Variables

            #endregion Variables

            #region Data Validation

            #endregion Data Validation

            try
            {
                // Setup the Progress Information
                m_Progress.TotalTasks = 1;
                m_Progress.TotalItems = 1;
                m_Progress.CurrentTask = 0;
                m_Progress.CurrentItem = 0;
                m_Cancel = false;

                m_Log.Debug(Properties.Resources.Execute_Started);

                // If the EventStart event has been subscribed to let's fire it.
                if (EventStart != null)
                    EventStart(this, new PluginEventArgs(m_Progress, "Starting the task.", null));

                DoWork();

                // If the EventStart event has been subscribed to let's fire it.
                if (EventEnd != null)
                {
                    if (m_Cancel)
                    {
                        EventEnd(this, new PluginEventArgs(m_Progress, "Task execution was cancelled.", null));
                    }
                    else
                    {
                        EventEnd(this, new PluginEventArgs(m_Progress, "Task execution is complete.", null));
                    }
                }

                

                // Exit 
                if (m_Cancel)
                {
                    m_Log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Cancelled);
                    return ExecuteResult.Cancelled;
                }
                else
                {
                    m_Log.DebugFormat(Properties.Resources.Execute_Ended, ExecuteResult.Ok);
                    return ExecuteResult.Ok;
                }
            }
            catch (Exception ex)
            {
                if (EventExceptionOccurred != null)
                    EventExceptionOccurred(this, new PluginEventArgs(m_Progress, "", ex));

                // Exit with an Exception
                return ExecuteResult.Exception;
            }

        }

        public bool Cancel()
        {
            m_Log.Debug(Properties.Resources.CancelMethodExecuted);
            m_Cancel = true;
            return true;
        }
        #endregion ITaskPlugin Methods

        #region ITaskPlugin Events
        public event OnStart EventStart;
        public event OnEnd EventEnd;
        public event OnProcessing EventProcessing;
        public event OnExceptionOccurred EventExceptionOccurred;
        #endregion ITaskPlugin Events

        #endregion

        #region Private Methods

        /// <summary>
        /// Read the list of commands from the configuration file and execute each of them.
        /// </summary>
        /// <returns>A <see cref="TaskPluginInterface.ExecuteResult"/>, with the result of the method call.</returns>
        private ExecuteResult DoWork()
        {

            #region Variables

            Process process = new Process();

            #endregion Variables

            // Setup the Progress Information
            m_Progress.TotalTasks = 1;
            m_Progress.TotalItems = config.CommandsToRun.Count;
            m_Progress.CurrentTask = 0;
            m_Progress.CurrentItem = 0;
            m_Cancel = false;

            m_Log.DebugFormat(Properties.Resources.CommandsFound, config.CommandsToRun.Count);

            foreach (ProcessStartInfo psi in config.CommandsToRun)
            {
                
                process.StartInfo = psi;
                process.Start();

                m_Log.DebugFormat(Properties.Resources.ExecutingCommand, process.StartInfo.FileName);

                // Update the current item
                m_Progress.CurrentItem++;
                if (EventProcessing != null)
                {
                    PluginEventArgs evt = new PluginEventArgs(m_Progress, "Working", null);
                    EventProcessing(this, evt);
                    if (evt.Cancel == true)
                    {
                        m_Log.Debug(Properties.Resources.CancelRequestRecieved);
                        m_Cancel = true;
                        break;
                    }
                }
            }
            
            // Everything Done
            if (m_Cancel)
            {
                return ExecuteResult.Cancelled;
            }
            else
            {
                m_Progress.CurrentTask++;
                return ExecuteResult.Ok;
            }

        }

        #endregion Private Methods

        #region Helper Functions

        #region LoadConfiguration

        private void LoadConfiguration()
        {
            string configFile = System.Reflection.Assembly.GetExecutingAssembly().Location + ".Settings.xml";
            config = new RunCommandConfig(null, configFile);

            // TODO: Figure out how to have different log4net instances for each
            //      plug-in.
            // Commented out until a fix can be determined to seperate the log files.
            // for each plug-in
            //if (!string.IsNullOrEmpty(config.Log4NetConfig))
            //{
            //    XmlDocument xml = new XmlDocument();
            //    xml.LoadXml(config.Log4NetConfig);
            //    log4net.Config.XmlConfigurator.Configure(xml.DocumentElement);
            //    xml = null;
            //}

            m_Schedule = TaskPluginInterface.ScheduleConfig.Load(config.Schedule);
            m_Log.Debug(Properties.Resources.ConfigurationLoaded);
        }
        #endregion LoadConfiguration

        #endregion Helper Functions

    }
}
