using System;
using System.Collections.Generic;
using System.Text;

namespace TaskPluginInterface
{
    /// <summary>
    /// The event arguments for the Plugin
    /// </summary>
    public class PluginEventArgs : EventArgs
    {
        #region Constructors

        /// <summary>
        /// Creates a new instance of the PluginEventArgs
        /// </summary>
        /// <param name="progress">A <see cref="Utilities.Progress"/> struct which indicates the
        /// current progress of the Plugin</param>
        /// <param name="message">A message to return back to the plug-in Host</param>
        /// <param name="raisedException">Any exception that might have been raised.</param>
        public PluginEventArgs(
           Utilities.Progress progress,
           string message,
           Exception raisedException)
        {
            this.m_Progress = progress;
            this.m_Message = message;
            this.m_RaisedException = raisedException;
        }


        #endregion Constructors

        #region Private Variables

        private Utilities.Progress m_Progress;
        private string m_Message = "";
        private Exception m_RaisedException;
        private bool m_Cancel = false;

        #endregion Private Variables

        #region Public Variables

        /// <summary>
        /// Indicates the current progress of the Plugin.
        /// </summary>
        public Utilities.Progress Progress
        {
            get { return m_Progress; }
        }
        
        /// <summary>
        /// A message returned from the Plugin
        /// </summary>
        public string Message
        {
            get { return m_Message; }
        }

        /// <summary>
        /// An exception raised from the Plugin
        /// </summary>
        public Exception RaisedException
        {
            get { return m_RaisedException; }
        }

        /// <summary>
        /// Indicates if the operation should be canceled.
        /// </summary>
        public bool Cancel
        {
            get { return m_Cancel;}
            set { m_Cancel = value;}
        }

        #endregion Public Variables

    }

}
